
#ifndef __AVLTREE_CPP
    #define __AVLTREE_CPP


    
    template<class T> AVLTreeNode<T>::AVLTreeNode(T content)
    {
        lnext = NULL;
        rnext = NULL;
        balance = 0;
        this->content = content;
    }

    template<class T> AVLTreeNode<T>::~AVLTreeNode()
    {
        if(lnext != NULL)
            delete lnext;
        if(rnext != NULL)
            delete rnext;
    }

    
    
    template<class T> AVLTree<T>::AVLTree()
    {
        root = NULL;
    }

    template<class T> AVLTree<T>::~AVLTree()
    {
        if(root != NULL)
            delete root;
    }
    
    
    
    template<class T> void AVLTree<T>::RR_rotation(AVLTreeNode<T>*& node)
    {
        AVLTreeNode<T>* rnexttemp = node->rnext;
        node->rnext = node->rnext->lnext;
        rnexttemp->lnext = node;
        node = rnexttemp;
        node->balance = 0;
        node->lnext->balance = 0;
    }

    template<class T> void AVLTree<T>::LL_rotation(AVLTreeNode<T>*& node)
    {
        AVLTreeNode<T>* lnexttemp = node->lnext;
        node->lnext = node->lnext->rnext;
        lnexttemp->rnext = node;
        node = lnexttemp;
        node->balance = 0;
        node->rnext->balance = 0;
    }

    template<class T> void AVLTree<T>::RL_rotation(AVLTreeNode<T>*& node)
    {
        AVLTreeNode<T>* ABak = node;
        AVLTreeNode<T>* BBak = node->rnext;
        node = node->rnext->lnext;
        AVLTreeNode<T>* BetaBak = node->lnext;
        AVLTreeNode<T>* GammaBak = node->rnext;
        node->lnext = ABak;
        node->rnext = BBak;
        ABak->rnext = BetaBak;
        BBak->lnext = GammaBak;
        
        if(node->balance == -1)
        {
            node->rnext->balance = 1;
            node->lnext->balance = 0;
        }
        else
        {
            node->rnext->balance = 0;
            node->lnext->balance = -1;
        }
        node->balance = 0;
    }

    template<class T> void AVLTree<T>::LR_rotation(AVLTreeNode<T>*& node)
    {
        AVLTreeNode<T>* ABak = node;
        AVLTreeNode<T>* BBak = node->lnext;
        node = node->lnext->rnext;
        AVLTreeNode<T>* BetaBak = node->lnext;
        AVLTreeNode<T>* GammaBak = node->rnext;
        node->rnext = ABak;
        node->lnext = BBak;
        ABak->lnext = GammaBak;
        BBak->rnext = BetaBak;
        
        if(node->balance == -1)
        {
            node->rnext->balance = 1;
            node->lnext->balance = 0;
        }
        else
        {
            node->rnext->balance = 0;
            node->lnext->balance = -1;
        }
        node->balance = 0;
    }



    template<class T> bool AVLTree<T>::Insert(AVLTreeNode<T>*& node, T insertion)
    {
        if(node == NULL)
        {
            node = new(AVLTreeNode<T>)(insertion);
            return 1;
        }
        if(insertion < node->content)
        {
            if(Insert(node->lnext,insertion))
                node->balance--;
            if(node->balance < -1)
            {
                if(node->lnext->balance == -1)
                    LL_rotation(node);
                else if(node->lnext->balance == 1)
                    LR_rotation(node);
            }
            return node->balance != 0;
        }
        if(insertion > node->content)
        {
            if(Insert(node->rnext,insertion))
                node->balance++;
            if(node->balance > 1)
            {
                if(node->rnext->balance == 1)
                    RR_rotation(node);
                else if(node->rnext->balance == -1)
                    RL_rotation(node);
            }
            return node->balance != 0;
        }
        return 0;
    }
    
    template<class T> bool AVLTree<T>::Insert(T insertion)
    {
        return Insert(root, insertion);
    }

    

    template<class T> void AVLTree<T>::Output(std::ostream& os, AVLTreeNode<T>*& node, int depth)
    {
        if(node == NULL)
            return;
        
        Output(os, node->lnext, depth+1);
        for(int i = 0; i < depth; i++)
            os << "   ";
        os << node->content << ":" << node->balance << "\n";
        Output(os, node->rnext, depth+1);
    }
    
    template<class T> void AVLTree<T>::Output(std::ostream& os)
    {
        Output(os, root, 0);
    }
    

#endif
